(ns login
  (:require-macros [cljs.core.async.macros :refer [go]])
  (:require [cljs.nodejs :as node]
            [macchiato.server :as moc]
            [cljs-http.client :as http]
            [cljs.core.async :refer [<!]]
            [cljs.core.async.interop :refer-macros [<p!]]
            [goog.crypt.base64 :as b64]))

(def open (node/require "open"))
(def inquirer (node/require "inquirer"))
(def fs (node/require "fs"))

(defn signup-next-steps-str []
  (str "#############################################\n"
       "Congratulations, you set Runops successfully!\n\n"
       "We created a mysql connection for you, so you can test Runops now:\n"
       "Hostname: 34.69.12.143\n"
       "Username: demo-user\n"
       "Password: GtK23d<ejkLy0ST2\n\n"
       "Next steps:\n"
       "#####################################################\n"
       "#### 1. Create a 'mysql 'target to run a script  ####\n"
       "#### 2. Execute a SQL script on that target      ####\n"
       "#####################################################\n\n"
       "#Tip: Targets can be of many types, including mysql, k8s,\n"
       "terraform, python, postgres, awscli, and many others.\n\n"
       "Run those in the terminal:\n"
       "$ runops targets create --name my-first-mysql-target --type mysql "
       "--secrets \"MYSQL_HOST=34.69.12.143,MYSQL_USER=demo-user,MYSQL_PASS=GtK23d<ejkLy0ST2\"\n"
       "$ runops tasks create -t my-first-mysql-target -s 'select * from demo.customers;'\n\n"
       "Please reach our docs to see all the power of Runops:\n"
       "https://docs.runops.io\n\n"
       "Press CTRL+C and run your first commands :)\n"))

;; server for callback
(defn handler [request response]
  (let [param-str (:query-string request)
        params (clojure.string/split param-str #"&")
        outcome-param (clojure.string/split (first params) #"=")
        outcome (clojure.string/replace (str (first outcome-param)) #"\?" "")
        outcome-value (str (second outcome-param))
        slack-param (clojure.string/split (second params) #"=")
        slack-value (second slack-param)]
    (if (= outcome "token")
      (do (when-not (.existsSync fs (str (.-HOME node/process.env) "/.runops"))
            (.mkdirSync fs (str (.-HOME node/process.env) "/.runops")))
          (.writeFileSync fs (str (.-HOME node/process.env) "/.runops/config") (str "Bearer " outcome-value))
          (response {:status 200
                     :body   "Runops login succeeded. You can close this tab now."})
          (js/console.log "Login succeeded !")
          (if slack-value
            (go (let [slack-url (b64/decodeString (str slack-value))
                      prompt (.prompt inquirer (clj->js [{:type "input" :name "slack" :message "Do you want to integrate with slack now?\n(1) Yes\n(2) No (you can do this later)\n"}]))
                      slack-decision (get (js->clj (<p! prompt)) "slack")]
                  (if (= "1" slack-decision)
                    (open slack-url))
                  (js/console.log (signup-next-steps-str))))
            (js/console.log (signup-next-steps-str))))
      (cond
        (= "unregistered" outcome-value) (do (js/console.log "Login failed! You are not registered at Runops. Please reach your org admin.")
                                             (response {:status 200
                                                        :body "Runops login failed! You are not registered at Runops. Please reach your org admin."}))

        (= "slack_not_configured" outcome-value) (do (js/console.log "You organisation have not configured an approval flow yet. Please reach your org admin.")
                                                     (response {:status 200
                                                                :body "Runops login failed! You organisation have not configured an approval flow yet. Please reach your org admin"}))

        (= "pending_review" outcome-value) (do (js/console.log "Your signup request was sent to your org Admin. You will receive an email when it is reviewed. Thanks.")
                                               (response {:status 200
                                                          :body "Your signup request was sent to your org Admin. You will receive an email when it is reviewed. Thanks."}))

        (= "email_mismatch" outcome-value) (do (js/console.log "Login failed! A different email was used. Please run 'runops logout' if you want to signup with a different email.")
                                               (response {:status 200
                                                          :body "Runops login failed. A different email was used. Please run 'runops logout' if you want to signup with a different email."}))

        (or (= "code_exchange_failure" outcome-value)
            (= "token_validation_failure" outcome-value)) (do (js/console.log "Login failed! An error happened during login. Please try again.")
                                                              (response {:status 200
                                                                         :body "Runops login failed. An error happened during login. Please try again.."}))))))


(defn server []
      (let [host "127.0.0.1"
            port 3000]
           (moc/start
             {:handler    handler
              :host       host
              :port       port
              :on-success #(prn "Opening the open the browser (^C to quit)")})))

;; login
(defn login [data]
  (server)
  (go (let [prompt (.prompt inquirer (clj->js [{:type "input" :name "email" :message "Enter your email:"}]))
            email (get (js->clj (<p! prompt)) "email")
            response (<! (http/get "https://api.runops.io/v1/login"
                                   {:query-params {:email email}}))]
        (open (:login_url (:body response))))))

(defn logout [data]
  (.writeFileSync fs (str (.-HOME node/process.env) "/.runops/config") "Bearer logged-out")
  (open "https://runops.us.auth0.com/v2/logout"))

;; signup
(defn signup [data]
  (server)
  (js/console.log "Welcome to Runops !")
  (go (let [prompt (.prompt inquirer (clj->js [{:type "input" :name "email" :message "Enter your email:"}]))
            email (get (js->clj (<p! prompt)) "email")
            response (<! (http/post "https://api.runops.io/v1/signup"
                                    {:json-params {:email email}}))]
        (js/console.log (str "Runops will try to open your browser now. If it fails, use the link below to continue:\n" (:login_url (:body response)) "\n\n"))
        (open (:login_url (:body response))))))